import React, { useState, useEffect } from 'react';
import { UserManager } from 'oidc-client';
export const AuthContext = React.createContext({
    signIn: /* istanbul ignore next */ () => { },
    signOut: /* istanbul ignore next */ () => { },
});
/**
 *
 * @param props AuthProviderProps
 */
export const AuthProvider = (props) => {
    const { children, autoSignIn = true, onSignIn, onSignOut, location = window.location, } = props;
    let { userManager } = props;
    const [userData, setUserData] = useState(null);
    if (!userManager) {
        const { authority, clientId, redirectUri } = props;
        userManager = new UserManager({
            authority,
            client_id: clientId,
            redirect_uri: redirectUri,
            silent_redirect_uri: redirectUri,
            post_logout_redirect_uri: redirectUri,
            response_type: 'code',
            scope: 'openid',
            loadUserInfo: true,
        });
    }
    /* istanbul ignore next */
    if (!userManager) {
        // This should never happen.
        throw new Error('UserManager is missing.');
    }
    const signIn = async () => {
        userManager && userManager.signinRedirect();
    };
    useEffect(() => {
        const getUser = async () => {
            const searchParams = new URLSearchParams(location.search);
            const hashParams = new URLSearchParams(location.hash.replace('#', '?'));
            if (searchParams.get('code') ||
                searchParams.get('id_token') ||
                searchParams.get('session_state') ||
                hashParams.get('code') ||
                hashParams.get('id_token') ||
                hashParams.get('session_state')) {
                await userManager.signinRedirectCallback();
                const user = await userManager.getUser();
                setUserData(user);
                onSignIn && onSignIn(user);
                return;
            }
            const user = await userManager.getUser();
            if ((!user || user === null) && autoSignIn) {
                signIn();
            }
            else {
                setUserData(user);
            }
            return;
        };
        getUser();
    }, [location]);
    const context = {
        signIn,
        signOut: async () => {
            await userManager.removeUser();
            setUserData(null);
            onSignOut && onSignOut();
        },
        userData,
    };
    return (React.createElement(AuthContext.Provider, { value: context }, children));
};
